#!/bin/sh

# this script works as a bootstrap import for AWS ami for MAG

export LD_LIBRARY_PATH=/opt/novell/lib:/opt/novell/lib64:/opt/novell/eDirectory/lib64:$LD_LIBRARY_PATH

NAM_DIR=/tmp/novell_access_manager/autoscale
#Create Flag file for scale down script to pick, in case of failure
FAILURE_FILE=$NAM_DIR/../failure

CURRENT_DIR=`dirname $0`
PARSER_DIR=/opt/novell/nam/autoscale/scripts

mkdir -p $NAM_DIR
LOG_OUTPUT="$NAM_DIR/ag_instance.log"
LOG_ERROR=$LOG_OUTPUT
#LOG_ERROR="$NAM_DIR/mag-import-log.err"
#ADMIN_PROPFILE="admin.properties"
IP_ADDR=""

#source "$NAM_DIR/$ADMIN_PROPFILE"
### Add entry in /etc/hosts
createHostEntry()
{
	echo "Creating host entry..."  >> $LOG_OUTPUT
	HOST_NAME=`hostname`
	IP_ADDR=`ip addr | grep -Po '(?!(inet 127.\d.\d.1))(inet \K(\d{1,3}\.){3}\d{1,3})' | head -n 1`
	HOST_PRESENT=`grep $HOST_NAME /etc/hosts`

	if [ -z "$HOST_PRESENT" ]
	then
        	echo -e "$IP_ADDR\t$HOST_NAME\t$HOST_NAME" >> /etc/hosts
	fi
}
EC2_INSTANCE_REGION=""
init_region()
{
	# get and parse instance region from instance metadata
	EC2_INSTANCE_REGION=$( curl -s http://169.254.169.254/latest/meta-data/placement/availability-zone )
	EC2_INSTANCE_REGION=$( echo "$EC2_INSTANCE_REGION" | sed 's|[a-z]$||g' )
	echo "region set to: $EC2_INSTANCE_REGION" >>$LOG_OUTPUT
}
get_secret_value()
{
	local secret_name=$1

	echo "reading the value for: $secret_name" >>$LOG_OUTPUT

	# read the values from secrets manager
	#echo `aws secretsmanager get-secret-value --secret-id "$secret_name" --region ap-south-1 --query SecretString --output text`

	# read the values from EC2 parameter store
	local value=`aws ssm get-parameter --name "$secret_name" --with-decryption --query Parameter.Value --output text --region "$EC2_INSTANCE_REGION"` 1>>$LOG_OUTPUT 2>>$LOG_ERROR
	echo $value

}

init_credentials()
{
	init_region

	ADMIN_USERNAME=$( get_secret_value AdminUsername )
	ADMIN_PASSWORD=$( get_secret_value AdminPassword )
	ADMIN_CONSOLE_IP=$( get_secret_value AdminConsolePrivateIP )
	MAG_CLUSTER_NAME=$( get_secret_value MAGClusterName )
	MAG_CLUSTER_ID=$( get_secret_value MAGClusterID )
	
	ADMIN_FQDN="cn=$ADMIN_USERNAME,o=novell"
	#echo "ADMIN_USERNAME=$ADMIN_USERNAME" >>$LOG_OUTPUT
	#echo "ADMIN_PASSWORD=$ADMIN_PASSWORD" >>$LOG_OUTPUT
	#echo "ADMIN_CONSOLE_IP=$ADMIN_CONSOLE_IP" >>$LOG_OUTPUT
	#echo "CLUSTER_NAME=$CLUSTER_NAME" >>$LOG_OUTPUT
	#echo "CLUSTER_ID=$CLUSTER_ID" >>$LOG_OUTPUT
}

getMagClusterID()
{
    response_code=$(curl --write-out %{http_code} --silent --output curl_response -k -X GET "https://$ADMIN_CONSOLE_IP:8443/amsvc/v1/agclusters" -u cn=$ADMIN_USERNAME,o=novell:$ADMIN_PASSWORD)
    if [[ ("$response_code" == 200) ]]; then
        json_body=$(<curl_response)
        parser_response=$(python -c 'execfile('"\"$PARSER_DIR/json_parser.py"\"'); get_ag_cluster_id()' "$json_body" "$LOG_OUTPUT" "$MAG_CLUSTER_NAME" 2>>$LOG_OUTPUT )
        if [[ ("$parser_response" != "failure") ]]; then
            clusterID=$parser_response
        fi
    fi
    rm -f curl_response
    echo $clusterID
}

validate_cluster(){
    expected_id=$(getMagClusterID)
    if [ -z "$expected_id" ]; then
        echo "Cluster Validation Failed : Cluster not found with given name - $MAG_CLUSTER_NAME" >> "$LOG_OUTPUT"
        echo "false"
    elif [[ "$expected_id" == "$MAG_CLUSTER_ID" ]]; then
        echo "Cluster Validation Successful : Cluster found with given name - $MAG_CLUSTER_NAME" >> "$LOG_OUTPUT"
        echo "true"
    else
        echo "Cluster Validation Failed : Cluster ID - $MAG_CLUSTER_ID in parameter store mismatches to that of Cluster name - $MAG_CLUSTER_NAME" >> "$LOG_OUTPUT"
        echo "false"
    fi
}

validate_credentials()
{
    if [[ "$(validate_cluster)" == "false" ]]; then
        echo "false"
    else
        echo "true"
    fi
}

reset_local_ip_in_server_xml()
{
	XMLMOD="/opt/novell/devman/jcc/utils/xmlmod"
	SERVER_XML="/opt/novell/nam/mag/conf/server.xml"

	LOCAL_PRIVATE_IP=$( curl -s http://169.254.169.254/latest/meta-data/local-ipv4 )

	${XMLMOD} ${SERVER_XML} "/Server/Service[1]/Connector[@NIDP_Name='connector']" "@address"
	${XMLMOD} ${SERVER_XML} "/Server/Service[1]/Connector[@NIDP_Name='connector']/@address" "${LOCAL_PRIVATE_IP}"

	${XMLMOD} ${SERVER_XML} "/Server/Service[1]/Connector[@protocol='HTTP/1.1' and @redirectPort='8443']" "@address"
	${XMLMOD} ${SERVER_XML} "/Server/Service[1]/Connector[@protocol='HTTP/1.1' and @redirectPort='8443']/@address" "${LOCAL_PRIVATE_IP}"
}

ec2_instance_abandon ()
{
     # get instance id from instance metadata
     EC2_INSTANCE_ID=$( curl -s http://169.254.169.254/latest/meta-data/instance-id )

     # get and parse instance region from instance metadata
     EC2_INSTANCE_REGION=$( curl -s http://169.254.169.254/latest/meta-data/placement/availability-zone )
     EC2_INSTANCE_REGION=$( echo "$EC2_INSTANCE_REGION" | sed 's|[a-z]$||g')

     # get the atuoscale group name
     AUTOSCALE_GROUPNAME=$( aws autoscaling describe-auto-scaling-instances --instance-ids $EC2_INSTANCE_ID --region $EC2_INSTANCE_REGION --query  AutoScalingInstances[0].AutoScalingGroupName --output text )

     #Get the life cylce launch hook name
     AUTOSCALE_LIFECYCLE_LAUNCH_HOOK=$( aws autoscaling describe-lifecycle-hooks --auto-scaling-group-name $AUTOSCALE_GROUPNAME --region $EC2_INSTANCE_REGION --query LifecycleHooks[0].LifecycleHookName --output text )

     #Abandon the instance

     aws autoscaling complete-lifecycle-action --lifecycle-action-result ABANDON  --instance-id $EC2_INSTANCE_ID  --lifecycle-hook-name $AUTOSCALE_LIFECYCLE_LAUNCH_HOOK --auto-scaling-group-name $AUTOSCALE_GROUPNAME --region $EC2_INSTANCE_REGION
}




ec2_instance_continue ()
{
     #get instance id from instance metadata
     EC2_INSTANCE_ID=$( curl -s http://169.254.169.254/latest/meta-data/instance-id  )
     # get and parse instance region from instance metadata
     EC2_INSTANCE_REGION=$( curl -s http://169.254.169.254/latest/meta-data/placement/availability-zone  )
     EC2_INSTANCE_REGION=$( echo "$EC2_INSTANCE_REGION" | sed 's|[a-z]$||g' )

     # get the atuoscale group name
     AUTOSCALE_GROUPNAME=$( aws autoscaling describe-auto-scaling-instances --instance-ids $EC2_INSTANCE_ID --region $EC2_INSTANCE_REGION --query    AutoScalingInstances[0].AutoScalingGroupName --output text )

     #Get the life cylce launch hook name
     AUTOSCALE_LIFECYCLE_LAUNCH_HOOK=$( aws autoscaling describe-lifecycle-hooks --auto-scaling-group-name $AUTOSCALE_GROUPNAME --region $EC2_INSTANCE_REGION --query LifecycleHooks[0].LifecycleHookName --output text )

     #Continue the instance
     aws autoscaling complete-lifecycle-action --lifecycle-action-result CONTINUE  --instance-id $EC2_INSTANCE_ID  --lifecycle-hook-name $AUTOSCALE_LIFECYCLE_LAUNCH_HOOK --auto-scaling-group-name $AUTOSCALE_GROUPNAME --region $EC2_INSTANCE_REGION

}

importMAG()
{
	echo "Importing the Access Gateway into the Admin Console..." >>  $LOG_OUTPUT
	USER=$ADMIN_FQDN
	PASSWD=$ADMIN_PASSWORD
	DEFAULT_IP=$ADMIN_CONSOLE_IP
	LOCAL_IP=$IP_ADDR


	JCC_DIR=/opt/novell/devman/jcc
	cd $JCC_DIR

	echo "Importing jcc..... " >>  $LOG_OUTPUT

	echo "Creating jccid.dat" >> $LOG_OUTPUT
	JCCID=`head -c8 /dev/urandom | od -An -tx8 | tr -d "[:space:]" | tr "[:lower:]" "[:upper:]"`
    echo $JCCID > /opt/novell/nam/mag/webapps/agm/WEB-INF/jccid.dat 
    echo "esp-$JCCID"  > /opt/novell/nesp/lib/webapp/WEB-INF/jccid.dat

	cat /opt/novell/nam/mag/webapps/agm/WEB-INF/jccid.dat >> $LOG_OUTPUT
	cat /opt/novell/nesp/lib/webapp/WEB-INF/jccid.dat >> $LOG_OUTPUT

	#echo "Java Import command: /opt/novell/java/bin/java -Djava.library.path=/opt/novell/lib64 -Djava.util.logging.config.file=./conf/conf-logging.properties -cp .:./lib/jcc.jar:./lib/NAMCommon.jar:./lib/certmgr.jar:./lib/npki.jar:./lib/jaxen-1.1.jar:./lib/dcc-core.jar com.novell.jcc.util.Configure --user $USER --pass ***** --localmgmtip $LOCAL_IP --remotemgmtip $DEFAULT_IP" >> $LOG_OUTPUT

	/opt/novell/java/bin/java -Djava.library.path=/opt/novell/lib64 -Djava.util.logging.config.file=./conf/conf-logging.properties -cp .:./lib/jcc.jar:./lib/NAMCommon.jar:./lib/certmgr.jar:./lib/npki.jar:./lib/jaxen-1.1.jar:./lib/dcc-core.jar com.novell.jcc.util.Configure --user $USER --pass $PASSWD --localmgmtip $LOCAL_IP --remotemgmtip $DEFAULT_IP 1>>$LOG_OUTPUT 2>>$LOG_ERROR

    jcc_import_status=$?
	if [[ $jcc_import_status -ne 0 ]]
	then
		IMPORT_STATUS=1
        echo "$IMPORT_STATUS"
        exit 0
	else	
        echo 'Stopping daemons...' >>  $LOG_OUTPUT
        /etc/init.d/novell-activemq stop >> $LOG_OUTPUT
        /etc/init.d/novell-agscd stop >> $LOG_OUTPUT
        /etc/init.d/novell-apache2 stop >> $LOG_OUTPUT
        /etc/init.d/novell-jcc stop >> $LOG_OUTPUT
        /etc/init.d/novell-mag stop  >> $LOG_OUTPUT

        #Removing the current configuration if any
        rm -rf /opt/novell/nam/mag/webapps/agm/WEB-INF/config/current/config.xml

        AGMID=`cat /opt/novell/nam/mag/webapps/agm/WEB-INF/jccid.dat`

        chown -R novlwww:novlwww /opt/novell/nam/mag
        chown -R root:root /opt/novell/nam/mag/bin


        echo "Check Access Java Command: CLASSPATH=.:/opt/novell/devman/jcc/utils/novbp.jar:/opt/novell/devman/jcc/utils/nids_install.jar:/opt/novell/devman/jcc/utils/vcdnbkup.jar /opt/novell/java/bin/java -Djava.library.path=/opt/novell/lib64 -cp .:/opt/novell/devman/jcc/utils/novbp.jar:/opt/novell/devman/jcc/utils/nids_install.jar:/opt/novell/devman/jcc/utils/vcdnbkup.jar com.novell.nids.install.CheckAccess $USER ****** $DEFAULT_IP" >> $LOG_OUTPUT

        CLASSPATH=.:/opt/novell/devman/jcc/utils/novbp.jar:/opt/novell/devman/jcc/utils/nids_install.jar:/opt/novell/devman/jcc/utils/vcdnbkup.jar /opt/novell/java/bin/java -Djava.library.path=/opt/novell/lib64 -cp .:/opt/novell/devman/jcc/utils/novbp.jar:/opt/novell/devman/jcc/utils/nids_install.jar:/opt/novell/devman/jcc/utils/vcdnbkup.jar com.novell.nids.install.CheckAccess $USER $PASSWD $DEFAULT_IP 1>>$LOG_OUTPUT 2>>$LOG_ERROR

        #echo "Java Import Command: /opt/novell/java/bin/java -Djava.library.path=/opt/novell/lib64 -Djava.util.logging.config.file=./conf/conf-logging.properties -cp .:./lib/jcc.jar:./lib/NAMCommon.jar:./lib/certmgr.jar:./lib/npki.jar:./lib/jaxen-1.1.jar:./lib/dcc-core.jar com.novell.jcc.util.Configure --user $USER --pass *****  --nidpid $NIDPID --tomcatdir /opt/novell/nam/idp --two_pass_configure --no_jcc_configure" 1>>$LOG_OUTPUT 2>>$LOG_ERROR

        /opt/novell/java/bin/java -Djava.library.path=/opt/novell/lib64 -Djava.util.logging.config.file=./conf/conf-logging.properties -cp .:./lib/jcc.jar:./lib/NAMCommon.jar:./lib/certmgr.jar:./lib/npki.jar:./lib/jaxen-1.1.jar:./lib/dcc-core.jar com.novell.jcc.util.Configure --user $USER --pass $PASSWD  --apacheid $AGMID --apachedir /opt/novell/apache2  --two_pass_configure --no_jcc_configure 1>>$LOG_OUTPUT 2>>$LOG_ERROR

        agm_import_status=$?
        if [[ $agm_import_status -ne 0 ]]
        then
            IMPORT_STATUS=1
            echo "$IMPORT_STATUS"
            exit 0
        else
            echo 'Starting daemons...' >> $LOG_OUTPUT
            /etc/init.d/novell-activemq restart >> $LOG_OUTPUT
            /etc/init.d/novell-agscd restart >> $LOG_OUTPUT
            /etc/init.d/novell-jcc restart >> $LOG_OUTPUT
            /etc/init.d/novell-mag restart >> $LOG_OUTPUT
            /etc/init.d/novell-apache2 restart >> $LOG_OUTPUT
        fi
    fi

IMPORT_STATUS=0
echo "$IMPORT_STATUS"

}

addMAGToCluster()
{
	echo "Adding MAG to the Cluster - $MAG_CLUSTER_NAME" >> $LOG_OUTPUT

	jcc_id=`cat /opt/novell/nam/mag/webapps/agm/WEB-INF/jccid.dat`
    
    local rest_url="https://$ADMIN_CONSOLE_IP:8443/amsvc/v1/agclusters/$MAG_CLUSTER_ID/devices/ag-$jcc_id"
	#echo "curl command: curl -k --ciphers ALL -u admin_username:admin_password -X POST \"$rest_url\"" >> $LOG_OUTPUT
	#curl -k --ciphers ALL -u $ADMIN_FQDN:$ADMIN_PASSWORD -X POST "$rest_url" 1>>$LOG_OUTPUT 2>>$LOG_ERROR

    #https://<AC_IP:PORT>/amsvc/v1/agclusters/<clusterID>/devices/<deviceID>
    local response_code=$(curl -X POST "$rest_url" -u $ADMIN_FQDN:$ADMIN_PASSWORD --write-out %{http_code} --silent --output curl_response -k --ciphers ALL)
    rm -f curl_response
    echo $response_code
}

updateMAGCluster()
{
	echo "Updating MAG Cluster - $MAG_CLUSTER_NAME" >>  $LOG_OUTPUT

	local rest_url="https://$ADMIN_CONSOLE_IP:8443/amsvc/v1/agclusters/$MAG_CLUSTER_ID"
	local rest_url_encoded=`echo $rest_url | sed "s# #%20#g"`

	#echo "curl command: curl -k --ciphers ALL -u admin_username:admin_password -X PUT -H \"Content-Type: application/json\" -d \"{\"update\": \"all\"}\" \"$rest_url_encoded\"" >> $LOG_OUTPUT
	curl -k --ciphers ALL -u $ADMIN_FQDN:$ADMIN_PASSWORD -X PUT -H "Content-Type: application/json" -d "{\"update\": \"all\"}" "$rest_url_encoded" 1>>$LOG_OUTPUT 2>>$LOG_ERROR

	if [ $? -ne 0 ]
	then
        	echo "Could not update MAG cluster - $MAG_CLUSTER_NAME" >>  $LOG_OUTPUT
	else
        	echo "Successfully updated MAG cluster - $MAG_CLUSTER_NAME" >>  $LOG_OUTPUT
	fi
}

getMagHeartBeat()
{
    resp=$(curl http://127.0.0.1:8181/nesp/app/heartbeat)
    if [[ ( "$resp" == "Success" ) ]]; then
    	echo "Success"
    else
	    echo "Failure"
    fi
}

checkForHeartBeat()
{
    state=$1
    SECONDS=0
    heartBeatResponse=$(getMagHeartBeat)
    while [[ ($SECONDS -lt 300) && ("$heartBeatResponse" != "Success") ]] ; do
       sleep 5
       heartBeatResponse=$(getMagHeartBeat)
    done

    if [[ ("$heartBeatResponse" != "Success") ]]
    then
        echo "MAG failed to respond to health check request..Abandoning the instance.." >> "$LOG_OUTPUT"
        if [[ ("$state" == "orphan") ]]; then
            echo "stage2" >> "$FAILURE_FILE"
        else
            echo "stage3" >> "$FAILURE_FILE"
        fi
        ec2_instance_abandon
        exit 3
    else
        echo "MAG is healthy...continue the instance" >> "$LOG_OUTPUT"
    fi
}

removeFiles()
{
	echo "Removing admin.properties" >>  $LOG_OUTPUT
	rm "$NAM_DIR/$ADMIN_PROPFILE"
}

######################################################################
### MAIN
######################################################################

###
cd $NAM_DIR
sleep 60
createHostEntry
init_credentials

if [[ "$(validate_credentials)" == "false" ]]; then
    echo "Failed : Validation of Parameter store details. Hence triggering abandon for this instance." >>  "$LOG_OUTPUT"
    echo "stage1" >> "$FAILURE_FILE"
    ec2_instance_abandon
    exit 1
fi

#reset_local_ip_in_server_xml   Looks like not need for  MAG
importMAGstatus=$(importMAG)
# Abandon the instance if the AG import is unsuccssfull.
if [ $importMAGstatus -ne 0  ]
   then
     echo "MAG import is failed...Abandoning the instance" >> $LOG_OUTPUT
     echo "stage1" >> "$FAILURE_FILE"
     ec2_instance_abandon
     exit 2
else
    echo " MAG import is completed...." >> $LOG_OUTPUT
fi

echo "Waiting till actual AG import is done" >> "$LOG_OUTPUT"

checkForHeartBeat "orphan"

#Add MAG to the cluster

responseCode=$(addMAGToCluster)

#Check for the MAG addition success else, abandon the instance

if [[ ("$responseCode" != 200) ]]; then
    echo "Failed adding MAG server to the Cluster - $MAG_CLUSTER_NAME" >> $LOG_OUTPUT
    echo "stage2" >> "$FAILURE_FILE"
    ec2_instance_abandon
    exit 4
else
   echo "Successfully added MAG server to the cluster - $MAG_CLUSTER_NAME " >> $LOG_OUTPUT
   ec2_instance_continue
fi

checkForHeartBeat "cluster"
