#!/bin/sh

#1 Check and validate Access Manager credentials
#2 Validate the Access Manager IDP cluster details
#3 Add the Access Manager credentials and cluster details to the AWS SSM parameter store
#4 Install the IDP

#sleep 60

source ./install_utility_functions.sh
source ./common_rpms_install.sh

PARSER_DIR=.
SCRIPT_DIR=..
LOG_FILE=/tmp/novell_access_manager/autoscale/ag_ami.log
MAIN_INSTALL_LOG=/tmp/novell_access_manager/autoscale/validate_remote_password.log

ADMIN_NAME=$1
ADMIN_PASS=$2
AC_IP=$3
#IDP_CLUSTER_NAME=$4
MAG_CLUSTER_NAME=$4
AWS_KMS_KEY=$5
AWS_S3REGION=$6
AWS_S3BUCKET=$7
PRIMARY_DEVICE_PRIVATE_IP=$8

EC2_INSTANCE_REGION=$( curl -s http://169.254.169.254/latest/meta-data/placement/availability-zone  )
EC2_INSTANCE_REGION=$( echo "$EC2_INSTANCE_REGION" | sed 's|[a-z]$||g' )

mkdir -p /tmp/novell_access_manager/autoscale
touch $MAIN_INSTALL_LOG
touch $LOG_FILE
> $LOG_FILE

cleanupJava()
{
    echo " cleaning up the setup for fresh install " >> $LOG_FILE
    rpm -e netiq-jrex
    rpm -e novell-base
    #rpm -e novell-jdk
    rm -rf /opt/novell/
    rm -rf /opt/netiq/
}

put_secret_value()
{
    local secret_name=$1
    local secret_value=$2

    echo " Writing the value for: $secret_name" >>$LOG_FILE
    #Write the values to EC2 SSM parameter store
    aws ssm put-parameter --name $secret_name --value $secret_value --type SecureString --key-id $AWS_KMS_KEY --overwrite --region $EC2_INSTANCE_REGION 1>>$LOG_FILE 2>>$LOG_FILE
}


getMagClusterID()
{
     response_code=$(curl --write-out %{http_code} --silent --output curl_response -k -X GET "https://$AC_IP:8443/amsvc/v1/agclusters" -u cn=$ADMIN_NAME,o=novell:$ADMIN_PASS)
    if [[ ("$response_code" == 200) ]]; then
        json_body=$(<curl_response)
        parser_response=$(python -c 'execfile('"\"$PARSER_DIR/json_parser.py"\"'); get_ag_cluster_id()' "$json_body" "$LOG_FILE" "$MAG_CLUSTER_NAME" 2>>$LOG_FILE )
        if [[ ("$parser_response" != "failure") ]]; then
            clusterID=$parser_response
        fi
    fi
    rm -f curl_response
    echo $clusterID
}


getPrimaryDeviceID()
{
     response_code=$(curl --write-out %{http_code} --silent --output curl_response -k -X GET "https://$AC_IP:8443/amsvc/v1/agclusters" -u cn=$ADMIN_NAME,o=novell:$ADMIN_PASS)
    if [[ ("$response_code" == 200) ]]; then
        json_body=$(<curl_response)
        parser_response=$(python -c 'execfile('"\"$PARSER_DIR/json_parser.py"\"'); get_primary_ag()' "$json_body" "$LOG_FILE" "$MAG_CLUSTER_NAME" 2>>$LOG_FILE )
        if [[ ("$parser_response" != "failure") ]]; then
            primaryDeviceID=$parser_response
        fi
    fi
    rm -f curl_response
    echo $primaryDeviceID
}


getPrimaryDeviceIP()
{
   clusterID=$1
   deviceID=$2
    response_code=$(curl --write-out %{http_code} --silent --output curl_response -k -X GET "https://$AC_IP:8443/amsvc/v1/agclusters/$clusterID/devices/$deviceID" -u cn=$ADMIN_NAME,o=novell:$ADMIN_PASS)
    if [[ ("$response_code" == 200) ]]; then
        json_body=$(<curl_response)
        parser_response=$(python -c 'execfile('"\"$PARSER_DIR/json_parser.py"\"'); get_primary_ag_ip()' "$json_body" "$LOG_FILE"  2>>$LOG_FILE )
        if [[ ("$parser_response" != "failure") ]]; then
            primaryDeviceIP=$parser_response
        fi
    fi
    rm -f curl_response
    echo $primaryDeviceIP

}


copyInstanceLogsToS3()
{
    echo "Backing up application log files to AWS S3 store" >> "$LOG_FILE"
    log_folder=/tmp/novell_access_manager
    instanceIP=`curl http://169.254.169.254/latest/meta-data/local-ipv4`
    tar_bundle=ag_ami_$(date +"%F")_$(date +"%s").tar.gz

    catalina_log=/opt/novell/nam/mag/logs/catalina.out
    jcc_logs=/opt/novell/devman/jcc/logs/jcc-0.log.0
    jcc_configure_logs=jcc_logs=/opt/novell/devman/jcc/logs/configure.log.0

    tar -czvf $tar_bundle $log_folder/* $catalina_log $jcc_logs $jcc_configure_logs

    aws s3 cp $tar_bundle s3://$AWS_S3BUCKET/logs/ --region $AWS_S3REGION

    rm -f $LOG_FILE $MAIN_INSTALL_LOG $tar_bundle
}

cleanupInstance()
{
find /root/.*history /home/*/.*history -exec rm -f {} \;
find / -name "authorized_keys" –exec rm –f {} \;
rm -rf /var/lib/cloud/instances/*
}

if [ ! -f  /opt/novell/java/bin/java ]
then
     install_java
fi

validateRemotePassword "cn=$ADMIN_NAME,o=novell $ADMIN_PASS $AC_IP"
#STATUS=`cat $MAIN_INSTALL_LOG`
if grep --quiet "Success" $MAIN_INSTALL_LOG
then
    cleanupJava

    MAG_CLUSTER_ID=$( getMagClusterID )
    if [ -z "$MAG_CLUSTER_ID" ]; then
        echo "Cluster not found with given name - $MAG_CLUSTER_NAME" >> $LOG_FILE
        copyInstanceLogsToS3
        exit 3
    fi
    echo "Found the cluster ID with given name, ID - $MAG_CLUSTER_ID" >> $LOG_FILE

    PRIMARY_DEVICE_ID=$( getPrimaryDeviceID )
    if [ -z "$PRIMARY_DEVICE_ID" ]; then
        echo "No Primary Device for for given - $MAG_CLUSTER_NAME" >> $LOG_FILE
        copyInstanceLogsToS3
        exit 3
    fi
    echo "Primary Device for the given cluster is - $PRIMARY_DEVICE_ID" >> $LOG_FILE
  
    PRIMARY_DEVICE_IP=$(getPrimaryDeviceIP "$MAG_CLUSTER_ID" "$PRIMARY_DEVICE_ID")
    if [ "$PRIMARY_DEVICE_PRIVATE_IP" != "$PRIMARY_DEVICE_IP" ]; then
        echo "Provided Primary Device IP "$PRIMARY_DEVICE_PRIVATE_IP" is not same as the actual Primary device IP -"$PRIMARY_DEVICE_IP" " >> $LOG_FILE
        copyInstanceLogsToS3
        exit 3
    fi
    echo "Primary Device IP for the given cluster is - $PRIMARY_DEVICE_IP" >> $LOG_FILE
   


    # Writing the NAM Secrets in the AWS SSM
    put_secret_value "AdminUsername" $ADMIN_NAME
    put_secret_value "AdminPassword" $ADMIN_PASS
    put_secret_value "AdminConsolePrivateIP" $AC_IP
    put_secret_value "MAGClusterName" $MAG_CLUSTER_NAME
    put_secret_value "MAGClusterID" $MAG_CLUSTER_ID
    put_secret_value "S3BucketName" $AWS_S3BUCKET
    put_secret_value "S3BucketRegion" $AWS_S3REGION

    #Modify the hostfile
    name=`hostname`
    ip=`curl http://169.254.169.254/latest/meta-data/local-ipv4`
    hostline=`echo $ip $name`
    echo $hostline >> /etc/hosts
    echo "created host entry." >> $LOG_FILE

    echo "Installing the MAG...." >> $LOG_FILE
    #Install the IDP
    cd $SCRIPT_DIR
    if [  -f "/etc/redhat-release" ]
    then
        echo -e "y""\n""n""\n""y""\n""n""\n""${AC_IP}""\n""${ADMIN_NAME}""\n""${ADMIN_PASS}""\n""${ADMIN_PASS}""\n" | sh ag_install.sh
    else
        echo -e "y""\n""n""\n""${AC_IP}""\n""${ADMIN_NAME}""\n""${ADMIN_PASS}""\n""${ADMIN_PASS}""\n" | sh ag_install.sh
    fi

    #Delete this MAG from the Admin Console
    echo "Deleting the MAG from the Admin Console...." >> $LOG_FILE
    sleep 60
    deviceID=`cat /opt/novell/nam/mag/webapps/agm/WEB-INF/jccid.dat`
    curl -k -X DELETE "https://${AC_IP}:8443/amsvc/v1/devices/ag/ag-$deviceID" -u cn=${ADMIN_NAME},o=novell:${ADMIN_PASS}
    sleep 60
    echo "MAG deleted from AC." >> $LOG_FILE
    copyInstanceLogsToS3
    cleanupInstance
    exit 0 ;
else
    echo " Provided credentials are wrong. Please check the AC IP / Admin Name / Admin Password " >> $LOG_FILE
    copyInstanceLogsToS3
    cleanupJava
    exit 1 ;
fi
